
% The code has to be modified for seed constraints. 
function [team, cc_score, lambda]= densest_subgraph(W, g, tol_score, debug)
%
%
%           densest_subgraph solves the following optimization problem 
%
%                                   <g_rest, f> + nu_S* max(f)                       vol_g(C)          
%         min             ---------------------------------------------  =    min  ------------  subject to S \in C    
%   f \in R^{n-|S|}_{+}     <deg_rest, f> - tv(f) + mu_S + <deg_S, f>          C     assoc(C)        
%
%     where g_rest is the weight vector restricted to 1:n - S
%           deg_rest is the degree vector of original W restricted to 1:n - S (but not the degree vector of the restricted graph)  
%           deg_S is the "outward" degree of the vertices, 1:n - S, to S
%           Note: assoc(C) = assoc(A) + assoc(C) + 2 cut(A, S) = vol(A) - cut(A, A') + assoc(C) + 2cut(A, S) 
%                          = vol(A) - cut(A, A'\S) - cut(A, S) + assoc(S) + 2 cut(A, S) 
%                          = vol(A) - cut(A, A'\S) + assoc(S) + cut(A, S) 
%           nu_S and mu_S are the volume and associations of subset S, respectively
%           tv(f) is the total variation of f on the graph W(1:n - S, 1:n - S)
%

%   Coding details:
%     Penalty terms for 
%       H_lbs'*f >= lbs :   sum( lbs - min(lbs, H_lbs'*f) ). 
%                           This is a difference of submodular functions and an upper bound is given by sum(lbs) - sum(SG_lbs'* f)
%                           where SG_lbs stores the subgradients for the second term.
%
%       H_ubs'*f <= ubs :   sum( max(0, H_ubs'*f - ubs) ). 
%                           This is a supermodular function. An upper bound is given by sum(SG_ubs'*f ),
%                           where SG_ubs stores the supergradients. 
%
%       dist <= d_0     :   <vol_deg_D, f> - tv_D(f),
%                           where D = max(0, dist - d_0) and deg_D is the degree vector of the weight matrix D
% (C)2012-13 Syama Sundar Rangapuram, Thomas Buehler and Matthias Hein 
% Machine Learning Group, Saarland University, Germany
% http://www.ml.uni-saarland.de

    % assert(sum(diag(W))==0, 'Diagonal entries of W are non-zero. '); % Diagonal entries need not be zero. This is taken care in optimal thresholding also!

    
    if ~exist('debug', 'var')
        debug = false;
    end
    
    S = []; % The code has to be modified for seed constraints. 
    J = S;    
    
    n = size(W, 1); f= rand(n, 1); %f = ones(n, 1);
    ind_rest = setdiff((1:n)', J);   
    
    ubs = 0; lbs = 0; H_lbs = sparse(n,1); H_ubs = sparse(n,1); dist = sparse(n, n); d_0 = 0;
    MAXITER = 1000; gamma = 0; norm_penalties = sparse(3,1); 
    
    [dix, djx, dval] = find(dist>d_0); D = sparse(dix, djx, dval - d_0, n, n); %D = max(0, dist - d_0);
    D_rest = D(ind_rest, ind_rest);    
    inner_obj_all = 0; eps_lambda = 1E-6;
    
    % RTM version had this : [ix,jx,wval]=find(W);

    W_rest = W(ind_rest, ind_rest);
    [ix_rest,jx_rest,wval_rest]=find(W_rest);

    W_ir_S = W(ind_rest,J);
    deg = sum(W,2); % we need the actual degree of vertices to compute cut(J,A) and association of A
    deg_S = sum(W_ir_S,2); % this is cut(J,A)
    deg_tilde = deg(ind_rest) + deg_S; % the first part is from vol(A) of vol(A) - cut(A,A') = assoc(A)

    mu_S = sum(sum(W(J,J))); nu_S = sum(g(J)); 

    f = f(ind_rest); g_rest = g(ind_rest); 

    lbs_rest = lbs-sum(H_lbs(J,:)); ubs_rest = ubs-sum(H_ubs(J,:)); H_lbs_rest = H_lbs(ind_rest, :); H_ubs_rest = H_ubs(ind_rest, :);
    dc = opt_thresh_forte_soft_cnstrs(W_rest, f, g_rest, lbs_rest, ubs_rest, H_lbs_rest, H_ubs_rest, gamma, mu_S, nu_S, W_ir_S, D_rest, norm_penalties);  % norm_penalties does not have a component for subset constraint
    [lambda, SG_lbs, SG_ubs] = objective_forte_soft_cnstrs(f, dc, gamma, length(f), max(0, lbs_rest), ubs_rest, wval_rest, ix_rest, jx_rest, g_rest, H_lbs_rest, H_ubs_rest, deg_tilde, mu_S, nu_S, D_rest, norm_penalties);  % norm_penalties does not have a component for subset constraint

    lambda_all=lambda;

    [Dix_rest, Djx_rest, Dval_rest] = find(D_rest); deg_D_rest = sum(D_rest,2);    
    converged=false; l=2;
    
    MAXITER_start = 50;
    while (~converged)

%   The inner problem takes the form:
%           min         <g_rest - lambda* deg_rest - lambda* deg_S, f> + nu_S * max(f) - labmda*mu_S* <sg_max, f> + gamma*sum(lbs) + gamma * sum( (SG_ubs' - SG_lbs')*f ) + gamma* <vol_deg_D - sg_tv_D, f> + lambda* tv(f)
%   f \in R^{n-|S|}_{+}
%
%   This simplifies to 
%           min         <g_rest - lambda* deg_rest - lambda* deg_S, f> + nu_S * max(f) - labmda*mu_S* max(f) + gamma*sum(lbs) + gamma * sum(SG_ubs' - SG_lbs')*f  + gamma* <vol_deg_D - sg_tv_D, f> + lambda* tv(f)
%   f \in R^{n-|S|}_{+}
%
%   This is rewritten as, using the shorthand given below,
%           min         <g_rest - lambda* (deg_tilde + mu_S* sg_max) + gamma * sum(SG_ubs - SG_lbs + vol_deg_D - sg_tv_D), f> + (nu_S + gamma*sum(lbs)) * max(f) + lambda* tv(f)
%   f \in R^{n-|S|}_{+}
%
%   shorthand used in the code:
%           deg_tilde = deg_rest + deg_S 
%           sg_max is a subgradient of max(f)
%           vec = - ( g_rest - lambda* (deg_tilde + mu_S*sg_max) + gamma*sum(SG_lbs) - gamma*sum(SG_ubs) - gamma*vol_deg_D + gamma*sg_tv_D )
%           c = nu_S + gamma*sum(lbs)

        %f_rest = f(ind_rest);
        [maxf, indmax] = max(f);
        sg_max = zeros(length(f), 1); sg_max(indmax) = 1;
            
        sg_tv_D = gamma*norm_penalties(end)*sparse(Dix_rest,1,Dval_rest.*sign(f(Dix_rest) - f(Djx_rest)), size(D_rest,1),1);
%         vec = sub+gamma*SG_lbs*norm_penalties(1:length(lbs))-...
%                 gamma*SG_ubs*norm_penalties(length(lbs)+1:length(lbs)+length(ubs)) - g_rest -...
%                 gamma*2*norm_penalties(end)*sum(D_rest,2) + lambda*(deg_tilde+mu_S*indmax);        
        
        vec = - g_rest + lambda*(deg_tilde + mu_S*sg_max) + gamma*SG_lbs*norm_penalties(1:length(lbs)) - gamma*SG_ubs*norm_penalties(length(lbs)+1:length(lbs)+length(ubs));
        vec = vec - gamma*norm_penalties(end)*deg_D_rest + sg_tv_D;
        
        c = nu_S + gamma*norm_penalties(1:length(lbs))'*lbs';

        if c == 0
            %[f_new1,beta1,primalobj1,dualobj1,iter1,f_new,primalobj,dualobj]=mex_forte_uncnstr_inner_problem(W_rest,2*vec,zeros(length(wval_rest),1),MAXITER,1E-8,4*lambda^2*max(sum(W_rest.^2)),lambda,1E-8, MAXITER_start);
            
            % We will solve the inner problem via s-t cuts
            T = sparse(n,2); T(:,1) = lambda*deg_tilde; T(:,2) = g_rest;
            [flow,labels] = maxflow(lambda*W,T);   % source side vertices have label 0!
            f_new = labels; f_new(labels==1) = 0; f_new(labels==0) = 1; f_new = double(f_new);
            dualobj = flow - lambda*sum(deg_tilde); 
            primalobj = dualobj; primal_obj = dualobj;
            
        else
            [f_new1,beta1,primalobj1,dualobj1,iter1,f_new,primalobj,dualobj]=mex_forte_soft_cnstrs_inner_problem(W_rest,2*vec,zeros(length(wval_rest),1),MAXITER,1E-8,4*lambda^2*max(sum(W_rest.^2)),lambda,1E-8,2*c, MAXITER_start);
        end
        
        
        %if primalobj==0
        if length(unique(labels))==1
            %ip_opt = true;
            f_new=f;
            lambda_new=lambda;
            SG_lbs_new=SG_lbs;
            SG_ubs_new=SG_ubs;
        else                                
            %primalobj = primalobj/norm(f_new); dualobj = 2*dualobj/norm(f_new); % the actual optimal is the unit vector; the mex funciton returns the non-unit vector.
                                                                                % the dual differs by a factor of 1/2 in the mex file.
            %f_new = f_new/norm(f_new); % the actual optimal is the unit vector; the mex funciton returns the non-unit vector.
            dc = opt_thresh_forte_soft_cnstrs(W_rest, f_new, g_rest, lbs_rest, ubs_rest, H_lbs_rest, H_ubs_rest, gamma, mu_S, nu_S, W_ir_S, D_rest, norm_penalties);  % norm_penalties does not have a component for subset constraint
            [lambda_new,SG_lbs_new,SG_ubs_new]=objective_forte_soft_cnstrs(f_new, dc, gamma, length(f_new), max(0, lbs_rest), ubs_rest, wval_rest, ix_rest, jx_rest, g_rest, H_lbs_rest, H_ubs_rest, deg_tilde, mu_S, nu_S, D_rest, norm_penalties);  % norm_penalties does not have a component for subset constraint
        end

        %primal_obj = -vec'*f_new + c * max(f_new) + lambda* 0.5* wval_rest'*abs( f_new(ix_rest) - f_new(jx_rest) );        
        if debug
            display(['flow = ', num2str(flow), ' primal_obj = ', num2str(primal_obj), ' dualobj = ', num2str(dualobj), ' lambda = ', num2str(lambda), ' lambda_new = ', num2str(lambda_new)]);
        end
        if ~isnan(lambda_new)
            if lambda_new > lambda
                if MAXITER_start >= MAXITER || iter1 >= MAXITER
                    f_new = f;  % We got a positive value and MAXITER exhausted.. so return the last iterate!
                    break;
                end
                MAXITER_start = iter1*2;
            end
        end
        
        % check if converged
%         diff=abs(lambda_new-lambda)/lambda;
%         if (diff<eps_lambda || lambda_new == 0 || lambda==0)
%         %if (diff<eps_lambda || abs(primalobj) <= 1e-5)
%             converged=true;
%         end

        % we check convergence in term of team score...
        if 1/lambda_new - 1/lambda < tol_score   
            converged = true;
        end

        % update variables
        lambda=lambda_new;
        f=f_new;
        SG_lbs=SG_lbs_new;
        SG_ubs=SG_ubs_new;

        inner_obj_all(l)=primalobj;
        lambda_all(l)=lambda_new;

        l=l+1;
    end

    if debug
        fprintf('  \n');
    end

    dc_temp = opt_thresh_forte_soft_cnstrs(W_rest, f_new, g_rest, lbs_rest, ubs_rest, H_lbs_rest, H_ubs_rest, gamma, mu_S, nu_S, W_ir_S, D_rest, norm_penalties);  % norm_penalties does not have a component for subset constraint
    
    lambda_dc = objective_forte_soft_cnstrs(dc_temp, dc_temp, gamma, length(f_new), max(0, lbs_rest), ubs_rest, wval_rest, ix_rest, jx_rest, g_rest, H_lbs_rest, H_ubs_rest, deg_tilde, mu_S, nu_S, D_rest, norm_penalties);
    lambda = objective_forte_soft_cnstrs(f_new, dc_temp, gamma, length(f_new), max(0, lbs_rest), ubs_rest, wval_rest, ix_rest, jx_rest, g_rest, H_lbs_rest, H_ubs_rest, deg_tilde, mu_S, nu_S, D_rest, norm_penalties);
    if debug
        if (lambda_dc - lambda) > 1e-6
            disp(['Final Iteration: Opt thresh seems not to produce descent - lambda: ', num2str(lambda), ' and lambda_dc: ', num2str(lambda_dc)]);
        else
            disp(['Final Iteration: Opt thresh does not create problem - lambda: ', num2str(lambda), ' and lambda_dc: ', num2str(lambda_dc)]);
        end
    end
    
    lambda = lambda_dc;
    dc_subset=zeros(size(W,1),1);
    dc_subset(J) = 1;
    density_subset = sum(sum(W(dc_subset>0, dc_subset>0)))/ sum(g(dc_subset>0));

    dc=ones(size(W,1),1);
    dc(ind_rest) = dc_temp>0;
    max_density = sum(sum(W(dc==1, dc==1)))/ sum(g(dc>0));

    if max_density < density_subset
        dc = dc_subset;
        max_density = density_subset;
    end

    team = dc>0;
    cc_score = max_density;

    if debug
        disp(['Found a team of size ', num2str(sum(team)), ' with a score of ', num2str(cc_score)]);        
    end
end
    

